@@ -9,7 +9,6 @@ import subprocess
 import traceback
 import hashlib
 import lzma
-from symbol import except_clause
 
 try:
     import bflb_path
@@ -58,6 +57,7 @@ else:
         "bl602": "bl602",
         "bl702": "bl702",
         "bl808": "bl808",
+        "bl606p": "bl808",
         "bl616": "bl616",
         "wb03" : "wb03",
     }
@@ -66,7 +66,7 @@ else:
     bl_factory_params_file_prefix = 'bl_factory_params_'
    
     
-boot2_dir = 'boot2_iap_v5.0'
+boot2_dir = 'boot2_iap_v5.3'
 ENABLE_HAIER = False
 
 
@@ -126,13 +126,15 @@ def generate_romfs_img(romfs_dir, dst_img_name):
             exe = os.path.join(app_path, 'utils/genromfs', 'genromfs_amd64')
         elif machine == 'armv7l':
             exe = os.path.join(app_path, 'utils/genromfs', 'genromfs_armel')
+        elif machine == 'arm64':
+            exe = os.path.join(app_path, 'utils/genromfs', 'genromfs_arm64')            
     if exe is None:
         bflb_utils.printf('NO supported genromfs exe for your platform!')
         return -1
     dir = os.path.abspath(romfs_dir)
     dst = os.path.abspath(dst_img_name)
     bflb_utils.printf('Generating romfs image %s using directory %s ... ' % (dst, dir))
-    return subprocess.call([exe, '-d', dir, '-f', dst])
+    return subprocess.call([exe, '-d', dir, '-f', dst, '-a', '64'])
 
 
 def check_pt_table(pt_parcel, file):
@@ -162,6 +164,14 @@ def check_pt_table(pt_parcel, file):
                     if file_size > pt_parcel['media_len']:
                         bflb_utils.printf("Error: media bin size is overflow with partition table")
                         return False
+                if "d0fw.bin" in file_name:
+                    if file_size > pt_parcel['fw_d0_len']:
+                        bflb_utils.printf("Error: d0fw bin size is overflow with partition table")
+                        return False
+                if "imtb.bin" in file_name:
+                    if file_size > pt_parcel['imtb_len']:
+                        bflb_utils.printf("Error: imtb bin size is overflow with partition table")
+                        return False                                        
                 i += 1
         except Exception as e:
             bflb_utils.printf(e)
@@ -243,10 +253,12 @@ class BflbIotTool(object):
         dict_xtal = gol.xtal_type[chiptype]
     
         if cpu_type is not None:
-            bootheader_section_name = "BOOTHEADER_" + cpu_type + "_CFG"
+            bootheader_section_name = "BOOTHEADER_" + cpu_type.upper() + "_CFG"
         else:
             if "BOOTHEADER_CPU0_CFG" in cfg.sections():
                 bootheader_section_name = "BOOTHEADER_CPU0_CFG"
+            elif "BOOTHEADER_GROUP0_CFG" in cfg.sections():
+                bootheader_section_name = "BOOTHEADER_GROUP0_CFG"
             else:
                 bootheader_section_name = "BOOTHEADER_CFG"
     
@@ -256,10 +268,13 @@ class BflbIotTool(object):
             else:
                 cfg.set(bootheader_section_name, 'boot2_enable', 0)
         if boot2 is True:
-            cfg.set(bootheader_section_name, 'img_start', '0x2000')
-            cfg.set(bootheader_section_name, 'cache_enable', '1')
-            if cpu_type is not None:
-                cfg.set(bootheader_section_name, 'halt_cpu1', '1')
+            if chiptype == "bl808":
+                cfg.set(bootheader_section_name, 'group_image_offset', '0x2000')
+            else:
+                cfg.set(bootheader_section_name, 'img_start', '0x2000')
+                cfg.set(bootheader_section_name, 'cache_enable', '1')
+                if cpu_type is not None:
+                    cfg.set(bootheader_section_name, 'halt_cpu1', '1')
             #cfg.set(bootheader_section_name, 'key_sel', '0')
         else:
             if chiptype == "bl60x":
@@ -299,6 +314,8 @@ class BflbIotTool(object):
         else:
             if "BOOTHEADER_CPU0_CFG" in cfg.sections():
                 bootheader_section_name = "BOOTHEADER_CPU0_CFG"
+            elif "BOOTHEADER_GROUP0_CFG" in cfg.sections():
+                bootheader_section_name = "BOOTHEADER_GROUP0_CFG"
             else:
                 bootheader_section_name = "BOOTHEADER_CFG"
     
@@ -326,11 +343,13 @@ class BflbIotTool(object):
                             publickey=None,
                             privatekey=None,
                             cfg_ini=None,
-                            cpu_type=None):
+                            group_type=None):
         efuse_file = os.path.join(chip_path, chipname, "efuse_bootheader/efusedata.bin")
         efuse_mask_file = os.path.join(chip_path, chipname, "efuse_bootheader/efusedata_mask.bin")
         if chiptype == "bl60x":
             bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader_cpu0.bin")
+        elif chiptype == "bl808":
+            bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader_group0.bin")
         else:
             bh_file = os.path.join(chip_path, chipname, "efuse_bootheader/bootheader.bin")
         cfg = BFConfigParser()
@@ -346,24 +365,26 @@ class BflbIotTool(object):
             f = cfg_ini
         cfg.read(f)
         if bintype == "fw":
-            if cpu_type is None:
+            if group_type is None or group_type == "Group0":
                 bootinfo_file = os.path.join(chip_path, chipname, "img_create_iot", "bootinfo.bin")
                 img_file = os.path.join(chip_path, chipname, "img_create_iot", "img.bin")
             else:
                 bootinfo_file = os.path.join(chip_path, chipname,
-                                             "img_create_iot", "bootinfo_{0}.bin".format(cpu_type.lower()))
+                                             "img_create_iot", "bootinfo_{0}.bin".format(group_type.lower()))
                 img_file = os.path.join(chip_path, chipname,
-                                        "img_create_iot", "img_{0}.bin".format(cpu_type.lower()))
+                                        "img_create_iot", "img_{0}.bin".format(group_type.lower()))
         else:
             bootinfo_file = os.path.join(chip_path, chipname,
                                          "img_create_iot", "bootinfo_{0}.bin".format(bintype))
             img_file = os.path.join(chip_path, chipname, "img_create_iot", "img_{0}.bin".format(bintype))
     
-        if cpu_type is not None:
-            img_section_name = "Img_" + cpu_type + "_Cfg"
+        if group_type is not None:
+            img_section_name = "Img_" + group_type + "_Cfg"
         else:
             if "Img_CPU0_Cfg" in cfg.sections():
                 img_section_name = "Img_CPU0_Cfg"
+            elif "Img_Group0_Cfg" in cfg.sections():
+                img_section_name = "Img_Group0_Cfg"
             else:
                 img_section_name = "Img_Cfg"
         cfg.set(img_section_name, 'boot_header_file', bh_file)
@@ -454,7 +475,9 @@ class BflbIotTool(object):
         fw_ota_bin_ver_haier = 'M_1.0.00-e-UDISCOVERY_UWT'
         fw_ota_bin_sn_haier = '00'*16
         fw_ota_bin_md5 = hashlib.md5(fw_ota_bin).hexdigest()      
-        fw_ota_bin_haier = bytes.fromhex(fw_ota_bin_ver_haier.encode('utf-8').hex() + '00'*7 + fw_ota_bin_sn_haier + fw_ota_bin_md5.encode('utf-8').hex() + version.encode('utf-8').hex() + '00'*(48-len(version)))        
+        fw_ota_bin_haier = bytes.fromhex(fw_ota_bin_ver_haier.encode('utf-8').hex() + \
+                           '00'*7 + fw_ota_bin_sn_haier + fw_ota_bin_md5.encode('utf-8').hex() + \
+                           version.encode('utf-8').hex() + '00'*(48-len(version)))        
         if dir_ota:
             fw_ota_path_haier = os.path.join(dir_ota, "FW_OTA_Haier.bin.ota")
         else:
@@ -473,7 +496,7 @@ class BflbIotTool(object):
     
         fw_ota_bin = bytearray()
         fw_ota_bin_xz = bytearray()
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin")
         else:
             fw_ota_path = os.path.join(ota_path, cpu_type + "_OTA.bin")
@@ -481,7 +504,7 @@ class BflbIotTool(object):
             file_bytes = bin_f.read()
             for b in file_bytes:
                 fw_ota_bin.append(b)
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin.xz")
         else:
             fw_ota_path = os.path.join(ota_path, cpu_type + "_OTA.bin.xz")
@@ -495,7 +518,7 @@ class BflbIotTool(object):
         fw_ota_bin_xz_ota = self.bl60x_mfg_ota_header(chipname, fw_ota_bin_xz, use_xz=1)
         for b in fw_ota_bin_xz:
             fw_ota_bin_xz_ota.append(b)
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin.xz.ota")
             fw_ota_path1 = os.path.join(ota_path, "FW_OTA.bin.hash")
         else:
@@ -519,7 +542,7 @@ class BflbIotTool(object):
         if os.path.isdir(ota_path) is False:
             os.mkdir(ota_path)
     
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             bootinfo_fw_path = os.path.join(chip_path, chipname, "img_create_iot", "bootinfo.bin")
         else:
             bootinfo_fw_path = os.path.join(chip_path, chipname,
@@ -534,7 +557,7 @@ class BflbIotTool(object):
             fw_ota_bin.append(0xFF)
             i = i - 1
         bflb_utils.printf("FW OTA bin header is Done. Len is %d" % len(fw_ota_bin))
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             img_fw_path = os.path.join(chip_path, chipname, "img_create_iot", "img.bin")
         else:
             img_fw_path = os.path.join(chip_path, chipname,
@@ -547,7 +570,7 @@ class BflbIotTool(object):
         if ENABLE_HAIER:
             self.bl60x_mfg_ota_header_haier(fw_ota_bin, chipname, version, dir_ota)
         fw_ota_bin_header = self.bl60x_mfg_ota_header(chipname, fw_ota_bin, use_xz=0)
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin")
         else:
             fw_ota_path = os.path.join(ota_path, cpu_type + "_OTA.bin")
@@ -556,7 +579,7 @@ class BflbIotTool(object):
     
         for b in fw_ota_bin:
             fw_ota_bin_header.append(b)
-        if cpu_type is None:
+        if cpu_type is None or cpu_type == "Group0":
             fw_ota_path = os.path.join(ota_path, "FW_OTA.bin.ota")
         else:
             fw_ota_path = os.path.join(ota_path, cpu_type + "_OTA.bin.ota")
@@ -585,6 +608,9 @@ class BflbIotTool(object):
             if chiptype == "bl60x":
                 efusefile = img_cfg.get("Img_CPU0_Cfg", "efuse_file")
                 efusemaskfile = img_cfg.get("Img_CPU0_Cfg", "efuse_mask_file")
+            elif chiptype == "bl808":
+                efusefile = img_cfg.get("Img_Group0_Cfg", "efuse_file")
+                efusemaskfile = img_cfg.get("Img_Group0_Cfg", "efuse_mask_file")
             else:
                 efusefile = img_cfg.get("Img_Cfg", "efuse_file")
                 efusemaskfile = img_cfg.get("Img_Cfg", "efuse_mask_file")
@@ -597,8 +623,8 @@ class BflbIotTool(object):
         else:
             self.efuse_load_en = False
         return img_create.create_sp_media_image_file(f, chiptype, cpu_type)
-    
-    
+
+
     def bl60x_mfg_flasher_cfg(self, uart, baudrate='57600', cfg_ini=None):
         cfg = BFConfigParser()
         if cfg_ini in [None, '']:
@@ -685,6 +711,8 @@ class BflbIotTool(object):
                                             pt_parcel,
                                             media,
                                             mfg,
+                                            d0fw=False,
+                                            imtb=False,
                                             flash_opt="1M"):
         bflb_utils.printf("========= eflash loader config =========")
         d_files = []
@@ -729,47 +757,79 @@ class BflbIotTool(object):
             d_files.append("chips/" + chipname + "/partition/partition.bin")
             d_addrs.append(hex(pt_parcel['pt_addr1'])[2:])
     
-        if bin_file is True and 'fw_cpu0_addr' in pt_parcel:
-            if bind_bootinfo is True:
-                whole_img = chipnamedir + "/img_create_iot/whole_img_cpu0.bin"
-                whole_img_len = 0x1000 + os.path.getsize(
-                    chipnamedir + "/img_create_iot/img_cpu0.bin")
-                whole_img_data = bl_create_flash_default_data(whole_img_len)
-                filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo_cpu0.bin"])[0]
-                whole_img_data[0:len(filedata)] = filedata
-                filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img_cpu0.bin"])[0]
-                whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
-                fp = open(whole_img, 'wb+')
-                fp.write(whole_img_data)
-                fp.close()
-                d_files.append("chips/" + chipname + "/img_create_iot/whole_img_cpu0.bin")
-                d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
-            else:
-                d_files.append("chips/" + chipname + "/img_create_iot/bootinfo_cpu0.bin")
-                d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
-                d_files.append("chips/" + chipname + "/img_create_iot/img_cpu0.bin")
-                d_addrs.append(hex(pt_parcel['fw_cpu0_addr'] + 0x1000)[2:])
-    
+        if bin_file is True:
+            if 'fw_cpu0_addr' in pt_parcel:
+                if bind_bootinfo is True:
+                    whole_img = chipnamedir + "/img_create_iot/whole_img_cpu0.bin"
+                    whole_img_len = 0x1000 + os.path.getsize(
+                        chipnamedir + "/img_create_iot/img_cpu0.bin")
+                    whole_img_data = bl_create_flash_default_data(whole_img_len)
+                    filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo_cpu0.bin"])[0]
+                    whole_img_data[0:len(filedata)] = filedata
+                    filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img_cpu0.bin"])[0]
+                    whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
+                    fp = open(whole_img, 'wb+')
+                    fp.write(whole_img_data)
+                    fp.close()
+                    d_files.append("chips/" + chipname + "/img_create_iot/whole_img_cpu0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
+                else:
+                    d_files.append("chips/" + chipname + "/img_create_iot/bootinfo_cpu0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_cpu0_addr'])[2:])
+                    d_files.append("chips/" + chipname + "/img_create_iot/img_cpu0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_cpu0_addr'] + 0x1000)[2:])
+            elif 'fw_group0_addr' in pt_parcel:
+                if bind_bootinfo is True:
+                    whole_img = chipnamedir + "/img_create_iot/whole_img_group0.bin"
+                    whole_img_len = 0x2000 + os.path.getsize(
+                        chipnamedir + "/img_create_iot/img_group0.bin")
+                    whole_img_data = bl_create_flash_default_data(whole_img_len)
+                    filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo_group0.bin"])[0]
+                    whole_img_data[0:len(filedata)] = filedata
+                    filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img_group0.bin"])[0]
+                    whole_img_data[0x2000:0x2000 + len(filedata)] = filedata
+                    fp = open(whole_img, 'wb+')
+                    fp.write(whole_img_data)
+                    fp.close()
+                    d_files.append("chips/" + chipname + "/img_create_iot/whole_img_group0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_group0_addr'])[2:])
+                else:
+                    d_files.append("chips/" + chipname + "/img_create_iot/bootinfo_group0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_group0_addr'])[2:])
+                    d_files.append("chips/" + chipname + "/img_create_iot/img_group0.bin")
+                    d_addrs.append(hex(pt_parcel['fw_group0_addr'] + 0x2000)[2:])
+
         if bin_file is True and 'fw_addr' in pt_parcel:
+            offset = 0x1000
+            if chiptype == "bl808":
+                offset = 0x2000
             if bind_bootinfo is True:
                 whole_img = chipnamedir + "/img_create_iot/whole_img.bin"
-                whole_img_len = 0x1000 + os.path.getsize(
+                whole_img_len = offset + os.path.getsize(
                     chipnamedir + "/img_create_iot/img.bin")
                 whole_img_data = bl_create_flash_default_data(whole_img_len)
                 filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/bootinfo.bin"])[0]
                 whole_img_data[0:len(filedata)] = filedata
                 filedata = bl_get_file_data(["chips/" + chipname + "/img_create_iot/img.bin"])[0]
-                whole_img_data[0x1000:0x1000 + len(filedata)] = filedata
+                whole_img_data[offset:offset + len(filedata)] = filedata
                 fp = open(whole_img, 'wb+')
                 fp.write(whole_img_data)
                 fp.close()
                 d_files.append("chips/" + chipname + "/img_create_iot/whole_img.bin")
-                d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
+                d_addrs.append(hex(pt_parcel['fw_addr'])[2:])                                    
             else:
                 d_files.append("chips/" + chipname + "/img_create_iot/bootinfo.bin")
                 d_addrs.append(hex(pt_parcel['fw_addr'])[2:])
                 d_files.append("chips/" + chipname + "/img_create_iot/img.bin")
-                d_addrs.append(hex(pt_parcel['fw_addr'] + 0x1000)[2:])
+                d_addrs.append(hex(pt_parcel['fw_addr'] + offset)[2:])
+                
+        if d0fw is True and pt_parcel['fw_d0_addr'] is not None:   
+            d_files.append("chips/" + chipname + "/img_create_iot/d0fw.bin")
+            d_addrs.append(hex(pt_parcel['fw_d0_addr'])[2:])   
+        
+        if imtb is True and pt_parcel['imtb_addr'] is not None:       
+            d_files.append("chips/" + chipname + "/img_create_iot/imtb.bin")
+            d_addrs.append(hex(pt_parcel['imtb_addr'])[2:])   
     
         conf_addr = pt_parcel.get('conf_addr', None)
         if ro_params is not None and len(ro_params) > 0 and conf_addr is not None:
@@ -777,7 +837,7 @@ class BflbIotTool(object):
                                                        chipnamedir + "/device_tree/ro_params.dtb")
             d_files.append("chips/" + chipname + "/device_tree/ro_params.dtb")
             d_addrs.append(hex(pt_parcel['conf_addr'])[2:])
-    
+             
         if media is True and pt_parcel['media_addr'] is not None:
             d_files.append("chips/" + chipname + "/img_create_iot/media.bin")
             d_addrs.append(hex(pt_parcel['media_addr'])[2:])
@@ -839,6 +899,8 @@ class BflbIotTool(object):
         pt_parcel = None
         media_bin_d = False
         mfg_d = False
+        d0_bin_d = False
+        imtb_bin_d = False
         boot2_en = False
         dts_bytearray = None
         
@@ -848,20 +910,10 @@ class BflbIotTool(object):
         self.config = config
 
         if chiptype == "bl702":
-            cfg_t = BFConfigParser()
-            cfg_t.read(self.bh_cfg_file)
-            if "BOOTHEADER_CPU0_CFG" in cfg_t.sections():
-                bootheader_section_name = "BOOTHEADER_CPU0_CFG"
-            else:
-                bootheader_section_name = "BOOTHEADER_CFG"
             if config["check_box"]['boot2_download'] is True:
                 boot2_en = False
-                cfg_t.set(bootheader_section_name, 'mfg_id', '0xff')
-                cfg_t.write(self.bh_cfg_file)
             else:
                 boot2_en = True
-                cfg_t.set(bootheader_section_name, 'mfg_id', '0xc2')
-                cfg_t.write(self.bh_cfg_file)
 
         cfg = BFConfigParser()
         cfg.read(self.eflash_loader_cfg)
@@ -882,6 +934,13 @@ class BflbIotTool(object):
                     bflb_utils.printf(error)
                     bflb_utils.set_error_code("0069")
                     return bflb_utils.errorcode_msg()
+            elif chiptype == "bl808" or chiptype == "bl616":
+                if bflb_flash_select.update_flash_cfg(chipname, chiptype, flash_id, self.bh_cfg_file, False,
+                                                      "BOOTHEADER_GROUP0_CFG") is False:
+                    error = "flash_id:" + flash_id + " do not support"
+                    bflb_utils.printf(error)
+                    bflb_utils.set_error_code("0069")
+                    return bflb_utils.errorcode_msg()
         else:
             error = "Do not find flash_id in eflash_loader_cfg.ini"
             bflb_utils.printf(error)
@@ -961,7 +1020,9 @@ class BflbIotTool(object):
         cfg_t = BFConfigParser()
         cfg_t.read(self.bh_cfg_file)
         if "BOOTHEADER_CPU0_CFG" in cfg_t.sections():
-                bootheader_section_name = "BOOTHEADER_CPU0_CFG"
+            bootheader_section_name = "BOOTHEADER_CPU0_CFG"
+        elif "BOOTHEADER_GROUP0_CFG" in cfg_t.sections():
+            bootheader_section_name = "BOOTHEADER_GROUP0_CFG"
         else:
             bootheader_section_name = "BOOTHEADER_CFG"
         crc_ignore = cfg_t.get(bootheader_section_name, 'crc_ignore')
@@ -971,7 +1032,10 @@ class BflbIotTool(object):
         cfg_t.write(self.bh_cfg_file)
 
         if config["check_box"]['boot2_download'] is True:
-            boot2 = config['input_path']['boot2_bin_input']
+            if chiptype == "bl808":
+                boot2 = "%s|UNUSED|UNUSED|" % config['input_path']['boot2_bin_input']
+            else:
+                boot2 = config['input_path']['boot2_bin_input']
             if boot2 is not None and len(boot2) > 0:
                 self.bl60x_fw_boot_head_gen(True, config['param'][chip_xtal], self.bh_cfg_file,
                                        config["check_box"]['encrypt'], config["check_box"]['sign'],
@@ -998,18 +1062,23 @@ class BflbIotTool(object):
         cfg_t.write(self.bh_cfg_file)
 
         if config["check_box"]['bin_download'] is True:
-            bin = config['input_path']['cfg2_bin_input']
+            if chiptype == "bl808":
+                bin = "%s|UNUSED|UNUSED|" % config['input_path']['cfg2_bin_input']
+            else:
+                bin = config['input_path']['cfg2_bin_input']
             if bin is not None and len(bin) > 0:
-                if parse_rfpa(bin) == b'BLRFPARA' and dts_bytearray:
+                temp = bin.split("|")[0]
+                if parse_rfpa(temp) == b'BLRFPARA' and dts_bytearray:
                     length = len(dts_bytearray)
-                    with open(bin, "rb") as fp:
+                    with open(temp, "rb") as fp:
                         bin_byte = fp.read()
                         bin_bytearray = bytearray(bin_byte)
                         bin_bytearray[1032:1032+length] = dts_bytearray
-                    filedir, ext = os.path.splitext(bin)
-                    bin = filedir + "_rfpa" + ext
-                    with open(bin, "wb") as fp:
+                    filedir, ext = os.path.splitext(temp)
+                    temp = filedir + "_rfpa" + ext
+                    with open(temp, "wb") as fp:
                         fp.write(bin_bytearray)
+                    bin = temp
                 self.bl60x_fw_boot_head_gen(False, config['param'][chip_xtal], self.bh_cfg_file,
                                        config["check_box"]['encrypt'], config["check_box"]['sign'],
                                        chipname, chiptype, cpu_type, boot2_en)
@@ -1018,6 +1087,26 @@ class BflbIotTool(object):
                                 config["input_path"]["publickey"], config["input_path"]["privatekey"])
                 self.bl60x_mfg_ota_bin_gen(chipname, chiptype, cpu_type, config["param"]["version"], config["param"]["ota"])
                 bin_d = True
+         
+        if 'd0_download' in config["check_box"] and 'd0_bin_input' in config['input_path']:    
+            if config["check_box"]['d0_download'] is True:
+                d0_bin = config['input_path']['d0_bin_input']
+                if d0_bin is not None and len(d0_bin) > 0:
+                    try:
+                        shutil.copyfile(d0_bin, os.path.join(chip_path, chipname, "img_create_iot", "d0fw.bin"))
+                    except Exception as e:
+                        bflb_utils.printf(e)
+                d0_bin_d = True
+        
+        if 'imtb_download' in config["check_box"] and 'imtb_bin_input' in config['input_path']:
+            if config["check_box"]['imtb_download'] is True:
+                imtb_bin = config['input_path']['imtb_bin_input']
+                if imtb_bin is not None and len(imtb_bin) > 0:
+                    try:
+                        shutil.copyfile(imtb_bin, os.path.join(chip_path, chipname, "img_create_iot", "imtb.bin"))
+                    except Exception as e:
+                        bflb_utils.printf(e)
+                imtb_bin_d = True
     
         if config["check_box"]['media_download'] is True:
             media_bin = config['input_path']['media_bin_input']
@@ -1042,7 +1131,10 @@ class BflbIotTool(object):
                 media_bin_d = True
     
         if config["check_box"]['mfg_download'] is True:
-            mfg = config['input_path']['mfg_bin_input']
+            if chiptype == "bl808":
+                mfg = "%s|UNUSED|UNUSED|" % config['input_path']['mfg_bin_input']
+            else:
+                mfg = config['input_path']['mfg_bin_input']
             if mfg is not None and len(mfg) > 0:
                 if parse_rfpa(mfg) == b'BLRFPARA' and dts_bytearray:
                     length = len(dts_bytearray)
@@ -1067,7 +1159,7 @@ class BflbIotTool(object):
                                 f)
                 mfg_d = True
         return self.bl60x_mfg_flasher_eflash_loader_cfg(chipname, chiptype, bin_d, boot2_d, ro_params_d,
-                                                   pt_parcel, media_bin_d, mfg_d)
+                                                   pt_parcel, media_bin_d, mfg_d, d0_bin_d, imtb_bin_d)
 
 
     def flasher_download_thread(self, chipname, chiptype, act, config, callback=None):
@@ -1088,7 +1180,7 @@ class BflbIotTool(object):
             return "no such action!"
         try:
             # config loader ini
-            if config['param']['interface_type'] == 'Uart':
+            if config['param']['interface_type'] == 'uart':
                 uart = config['param']['comport_uart']
                 uart_brd = config['param']['speed_uart']
                 if not uart_brd.isdigit():
@@ -1097,7 +1189,7 @@ class BflbIotTool(object):
                     return error                    
                 bflb_utils.printf("========= Interface is Uart =========")
                 self.bl60x_mfg_flasher_cfg(uart, uart_brd)
-            elif config['param']['interface_type'] == 'JLink':
+            elif config['param']['interface_type'] == 'jlink':
                 jlink_brd = config['param']['speed_jlink']
                 if not jlink_brd.isdigit():
                     error = '{"ErrorCode":"FFFF","ErrorMsg":"BAUDRATE MUST BE DIGIT"}'
@@ -1105,7 +1197,7 @@ class BflbIotTool(object):
                     return error     
                 bflb_utils.printf("========= Interface is JLink =========")
                 self.bl60x_mfg_flasher_jlink_cfg(rate=jlink_brd)
-            elif config['param']['interface_type'] == 'CKLink':
+            elif config['param']['interface_type'] == 'cklink':
                 bflb_utils.printf("========= Interface is CKLink =========")
                 self.bl60x_mfg_flasher_cklink_cfg()
             else:
@@ -1139,7 +1231,7 @@ class BflbIotTool(object):
                 cfg.set('FLASH_CFG', 'address', addrs_str)
                 cfg.write(self.eflash_loader_cfg, 'w')
                 self.eflash_loader_t = eflash_loader.BflbEflashLoader(chipname, chiptype)
-                if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'Uart':
+                if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'uart':
                     error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
                     bflb_utils.printf(error)
                     return error
@@ -1147,7 +1239,7 @@ class BflbIotTool(object):
                     self.bl60x_mfg_flasher_erase_all(True)
                 else:
                     self.bl60x_mfg_flasher_erase_all(False)
-                if config['param']['interface_type'] == 'Uart':
+                if config['param']['interface_type'] == 'uart':
                     options = ["--write", "--flash", "-p", uart]
                     args = parser_eflash.parse_args(options)
                     error = self.eflash_loader_t.efuse_flash_loader(args, self.eflash_loader_cfg,
@@ -1165,7 +1257,7 @@ class BflbIotTool(object):
                     # burn into flash
                     self.eflash_loader_t = eflash_loader.BflbEflashLoader(chipname, chiptype)
                     self.eflash_loader_t.set_config_file(self.bh_cfg_file, "")
-                    if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'Uart':
+                    if not config["param"]["comport_uart"] and config['param']['interface_type'] == 'uart':
                         error = '{"ErrorCode":"FFFF","ErrorMsg":"BFLB INTERFACE HAS NO COM PORT"}'
                         bflb_utils.printf(error)
                         return error
@@ -1182,7 +1274,7 @@ class BflbIotTool(object):
                             config["check_box"]["sign"] is True:
                             cfg_file = os.path.join(chip_path, chipname, "img_create_iot", "img_create_cfg.ini")
                             options.extend(["--createcfg=" + cfg_file])
-                    if config['param']['interface_type'] == 'Uart':
+                    if config['param']['interface_type'] == 'uart':
                         options.extend(["-p", uart])
                     args = parser_eflash.parse_args(options)
                     error = self.eflash_loader_t.efuse_flash_loader(args, self.eflash_loader_cfg,
@@ -1241,7 +1333,7 @@ def iot_download_cmd(args):
     config["param"] = dict()
     config["check_box"] = dict()
     config["input_path"] = dict()
-    config["param"]["interface_type"] = args.interface.capitalize()
+    config["param"]["interface_type"] = args.interface.lower()
     config["param"]["comport_uart"] =  args.port
     config["param"]["speed_uart"] = str(args.baudrate)
     config["param"]["speed_jlink"] = str(args.baudrate)
