@@ -108,6 +108,7 @@ else:
         "bl602": "bl602",
         "bl702": "bl702",
         "bl808": "bl808",
+        "bl606p": "bl808",
         "bl616": "bl616",
         "wb03" : "wb03",
     }
@@ -128,6 +129,7 @@ if conf_sign:
         "000a": "FLASH_WRITE_STATUS_REG_ERROR",
         "000b": "FLASH_DECOMPRESS_WRITE_ERROR",
         "000c": "FLASH_WRITE_XZ_ERROR",
+        "000d": "FLASH_SWITCH_BANK_ERROR",
         "0101": "CMD_ID_ERROR",
         "0102": "CMD_LEN_ERROR",
         "0103": "CMD_CRC_ERROR",
@@ -171,6 +173,24 @@ if conf_sign:
         "0406": "EFUSE_READ_ERROR",
         "0407": "EFUSE_READ_MAC_ERROR",
         "0408": "EFUSE_WRITE_MAC_ERROR",
+        "0501": "MEMORY_WRITE_PARA_ERROR",
+        "0502": "MEMORY_WRITE_ADDR_ERROR",
+        "0503": "MEMORY_WRITE_ERROR",
+        "0504": "MEMORY_READ_PARA_ERROR",
+        "0505": "MEMORY_READ_ADDR_ERROR",
+        "0506": "MEMORY_READ_ERROR",
+        "0508": "REG_WRITE_PARA_ERROR",
+        "0509": "REG_WRITE_ADDR_ERROR",
+        "050a": "REG_WRITE_ERROR",
+        "050b": "REG_READ_PARA_ERROR",
+        "050c": "REG_READ_ADDR_ERROR",
+        "050d": "REG_READ_ERROR",
+        "0601": "ECDH_PARA_ERROR",
+        "0602": "ECDH_PRIVATE_KEY_ERROR",
+        "0603": "ECDH_SHARED_KEY_ERROR",
+        "0604": "ECDH_RANDOM_VAL_ERROR",
+        "0605": "ECDH_DECRYPT_ERROR",
+        "0606": "ECDH_ENCRYPT_ERROR",
         "fffc": "PLL_ERROR",
         "fffd": "INVASION_ERROR",
         "fffe": "POLLING",
@@ -216,6 +236,7 @@ if conf_sign:
         "0040": "FLASH CFG NOT FIT WITH BOOTHEADER",
         "0041": "FLASH LOAD ROMFS FAIL",
         "0042": "FLASH SWITCH BANK FAIL",
+        "0043": "FLASH IDENTIFY FAIL",
         "0050": "IMG LOAD SHAKEHAND FAIL",
         "0051": "IMG LOAD BOOT CHECK FAIL",
         "0060": "IMG CREATE FAIL",
@@ -247,6 +268,8 @@ if conf_sign:
         "0080": "ROMFS NOT SET",
         "0081": "MFG BIN NOT SET",
         "0082": "PT CHECK FAIL",
+        "0083": "D0 FW BIN NOT SET",  
+        "0084": "IMTB BIN NOT SET",      
         "FFFF": "BURN RETRY FAIL",
     }
 else:
@@ -264,6 +287,7 @@ else:
         "000a": "BFLB_FLASH_WRITE_STATUS_REG_ERROR",
         "000b": "BFLB_FLASH_DECOMPRESS_WRITE_ERROR",
         "000c": "BFLB_FLASH_WRITE_XZ_ERROR",
+        "000d": "BFLB_FLASH_SWITCH_BANK_ERROR",
         "0101": "BFLB_CMD_ID_ERROR",
         "0102": "BFLB_CMD_LEN_ERROR",
         "0103": "BFLB_CMD_CRC_ERROR",
@@ -307,6 +331,24 @@ else:
         "0406": "BFLB_EFUSE_READ_ERROR",
         "0407": "BFLB_EFUSE_READ_MAC_ERROR",
         "0408": "BFLB_EFUSE_WRITE_MAC_ERROR",
+        "0501": "BFLB_MEMORY_WRITE_PARA_ERROR",
+        "0502": "BFLB_MEMORY_WRITE_ADDR_ERROR",
+        "0503": "BFLB_MEMORY_WRITE_ERROR",
+        "0504": "BFLB_MEMORY_READ_PARA_ERROR",
+        "0505": "BFLB_MEMORY_READ_ADDR_ERROR",
+        "0506": "BFLB_MEMORY_READ_ERROR",
+        "0508": "BFLB_REG_WRITE_PARA_ERROR",
+        "0509": "BFLB_REG_WRITE_ADDR_ERROR",
+        "050a": "BFLB_REG_WRITE_ERROR",
+        "050b": "BFLB_REG_READ_PARA_ERROR",
+        "050c": "BFLB_REG_READ_ADDR_ERROR",
+        "050d": "BFLB_REG_READ_ERROR",
+        "0601": "BFLB_ECDH_PARA_ERROR",
+        "0602": "BFLB_ECDH_PRIVATE_KEY_ERROR",
+        "0603": "BFLB_ECDH_SHARED_KEY_ERROR",
+        "0604": "BFLB_ECDH_RANDOM_VAL_ERROR",
+        "0605": "BFLB_ECDH_DECRYPT_ERROR",
+        "0606": "BFLB_ECDH_ENCRYPT_ERROR",
         "fffc": "BFLB_PLL_ERROR",
         "fffd": "BFLB_INVASION_ERROR",
         "fffe": "BFLB_POLLING",
@@ -352,6 +394,7 @@ else:
         "0040": "BFLB FLASH CFG NOT FIT WITH BOOTHEADER",
         "0041": "BFLB FLASH LOAD ROMFS FAIL",
         "0042": "BFLB FLASH SWITCH BANK FAIL",
+        "0043": "BFLB FLASH IDENTIFY FAIL",
         "0050": "BFLB IMG LOAD SHAKEHAND FAIL",
         "0051": "BFLB IMG LOAD BOOT CHECK FAIL",
         "0060": "BFLB IMG CREATE FAIL",
@@ -383,6 +426,8 @@ else:
         "0080": "BFLB ROMFS NOT SET",
         "0081": "BFLB MFG BIN NOT SET",
         "0082": "BFLB PT CHECK FAIL",
+        "0083": "D0 FW BIN NOT SET",  
+        "0084": "IMTB BIN NOT SET",        
         "FFFF": "BFLB BURN RETRY FAIL",
     }
 
